/*
 * @(#)DuoDerivedColor.java	1.4 08/02/04
 *
 * Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.sun.java.swing.plaf.nimbus;

import java.awt.Color;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * DuoDerivedColor - Is a derived color that is derived of two parent UIManager
 * defaults colors and offets and a value of the position between the two
 * colors. This is designed for use in gradients that need to have a calculated
 * color that is at some mid point between two derived colors. Property change
 * events are fired for the "rgb" property when the derived color changes.
 *
 * @author Jasper Potts
 */
class DuoDerivedColor extends Color implements PropertyChangeListener {
    private final Color color1, color2;
    private final float midPoint;
    private int argbValue;

    DuoDerivedColor(Color color1, Color color2, float midPoint) {
        super(0);
        this.color1 = color1;
        this.color2 = color2;
        this.midPoint = midPoint;
        if (color1 instanceof DerivedColor)
            ((DerivedColor)color1).addPropertyChangeListener(this);
        if (color2 instanceof DerivedColor)
            ((DerivedColor)color2).addPropertyChangeListener(this);
        rederiveColor();
    }

    public void propertyChange(PropertyChangeEvent evt) {
        rederiveColor();
    }

    /**
     * Recalculate the derived color from the UIManager parent color and
     * offsets
     */
    public void rederiveColor() {
        int r = color1.getRed() +
                (int) ((color2.getRed() - color1.getRed()) * midPoint + 0.5f);
        int g = color1.getGreen() +
                (int) ((color2.getGreen() - color1.getGreen()) * midPoint +
                        0.5f);
        int b = color1.getBlue() +
                (int) ((color2.getBlue() - color1.getBlue()) * midPoint + 0.5f);
        int a = color1.getAlpha() +
                (int) ((color2.getAlpha() - color1.getAlpha()) * midPoint +
                        0.5f);
        argbValue = ((a & 0xFF) << 24) |
                ((r & 0xFF) << 16) |
                ((g & 0xFF) << 8) |
                (b & 0xFF);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof DuoDerivedColor)) return false;
        DuoDerivedColor that = (DuoDerivedColor) o;
        if (Float.compare(that.midPoint, midPoint) != 0) return false;
        if (color1 != null ? !color1.equals(that.color1) : that.color1 != null)
            return false;
        if (color2 != null ? !color2.equals(that.color2) : that.color2 != null)
            return false;
        return true;
    }

    @Override
    public int hashCode() {
        int result = (color1 != null ? color1.hashCode() : 0);
        result = 31 * result + (color2 != null ? color2.hashCode() : 0);
        result = 31 * result + midPoint != +0.0f ?
                Float.floatToIntBits(midPoint) : 0;
        return result;
    }

    /**
     * Returns the RGB value representing the color in the default sRGB {@link
     * java.awt.image.ColorModel}. (Bits 24-31 are alpha, 16-23 are red, 8-15
     * are green, 0-7 are blue).
     *
     * @return the RGB value of the color in the default sRGB
     *         <code>ColorModel</code>.
     * @see java.awt.image.ColorModel#getRGBdefault
     * @see #getRed
     * @see #getGreen
     * @see #getBlue
     */
    @Override public int getRGB() {
        return argbValue;
    }
    
    /**
     * UIResource version of duo derived color
     */
    static class UIResource extends DuoDerivedColor implements javax.swing.plaf.UIResource {
        UIResource(Color c1, Color c2, float midPoint) {
            super(c1, c2, midPoint);
        }
    }
}
